@@ -48,6 +48,7 @@ import (
 	"github.com/prometheus/prometheus/discovery/scaleway"
 	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/prometheus/prometheus/discovery/triton"
+	"github.com/prometheus/prometheus/discovery/xds"
 	"github.com/prometheus/prometheus/discovery/zookeeper"
 	"github.com/prometheus/prometheus/pkg/labels"
 	"github.com/prometheus/prometheus/pkg/relabel"
@@ -439,6 +440,26 @@ var expectedConf = &Config{
 				},
 			},
 		},
+		{
+			JobName: "service-kuma",
+
+			HonorTimestamps: true,
+			ScrapeInterval:  model.Duration(15 * time.Second),
+			ScrapeTimeout:   DefaultGlobalConfig.ScrapeTimeout,
+
+			MetricsPath:      DefaultScrapeConfig.MetricsPath,
+			Scheme:           DefaultScrapeConfig.Scheme,
+			HTTPClientConfig: config.DefaultHTTPClientConfig,
+
+			ServiceDiscoveryConfigs: discovery.Configs{
+				&xds.KumaSDConfig{
+					Server:           "http://kuma-control-plane.kuma-system.svc:5676",
+					HTTPClientConfig: config.DefaultHTTPClientConfig,
+					RefreshInterval:  model.Duration(15 * time.Second),
+					FetchTimeout:     model.Duration(2 * time.Minute),
+				},
+			},
+		},
 		{
 			JobName: "service-marathon",
 
